% Computing the worst-case input belonging to the worst-case scheduling
% trajectory constraucted by the lower bound algorithm.

close all; clear all;


%% Select the datafile containing the worst-case parameter trajectory and the lower bound

%file='harald_pwl_03';
%file='harald_msin_10';
%file='catws_10_02'
%file='catws_13_02'
%file='catws_20_01'
file='rotated_08';

directory='./matfiles/';

load([directory file '.mat'],'c','gL','ptrajectory','tdata','lpvsys'); g=gL;




%% The algorithm starts here

options = odeset('RelTol',1e-9,'AbsTol',1e-12);
h=ptrajectory([],c,tdata,-1); w=2*pi/h;
[nx,nw,nz]=lpvsys([]); 



% Computing the norm of Cg/(eye(nx)-Ag)*Bg 
% ----------------------------------------
[nu,Z0,Xh,Yh,Ag,Bg,Cg] = obj_nu(c,g,lpvsys,ptrajectory,tdata);
norm(Cg/(eye(nx)-Ag)*Bg)
nu


% Computing W
% -----------
% Determining W as a point solution of Colaneri's Hamiltonian system
Efh = @(t,Evec) HColaneri(t,Evec,lpvsys,ptrajectory,g,c,tdata);
E0 = eye(2*nx); 
[~,Evec]=ode45(Efh, [0 h],E0(:),options);
Eh=reshape(Evec(end,:),2*nx,2*nx);
Wint=Eh

% Determining W from Q
% Computing Q from Xh,Yh,Z0
Q=[inv(Xh') -(Xh')\Z0;Yh/(Xh') Xh-Yh/(Xh')*Z0]


% Computing Q by integrating Cantoni's Hamiltonian system
Efh = @(t,Evec) HCantoni(t,Evec,lpvsys,ptrajectory,g,c,tdata);
E0 = eye(2*nx); 
[~,Evec]=ode45(Efh, [0 h],E0(:),options);
Eh=reshape(Evec(end,:),2*nx,2*nx);
Qint=Eh


% Transformation matrix
T=[zeros(nx) eye(nx);g*eye(nx) zeros(nx)];
WQ=T\Q*T
WQint=T\Qint*T


% Comparing the different solutions
abs(Wint-WQ)
abs(Wint-WQint)
abs(WQ-WQint)

% Picking one of them
% W=WQint;
W=WQ %we are using W computed from Cantoni's Q


% Finding an eigenvalue on the unit disk
% --------------------------------------
% Finding the unit modulus eigenvalue and the corresponding eigenvector.
[V,E]=eig(W);
e=diag(E)
abs(e)
i=find(abs(abs(e)-1)<1e-5,1);
thetap0=V(:,i);
eigval=e(i)


abs(eigval)
eigval=eigval/abs(eigval) %just for numerical reasons



% Computing the worst-case input and output for 1 period (can be complex!)
% -----------------------------------------------------------------------
options = odeset('RelTol',1e-6,'AbsTol',1e-9);
Hfh = @(t,thetap) HColaneri(t,thetap,lpvsys,ptrajectory,g,c,tdata);
[thetap.time,thetap.val]=ode45(Hfh,[0 h],thetap0,options);

N=length(thetap.time);

wwc.val=zeros(N,nw); wwc.time=thetap.time;
zwc.val=zeros(N,nz); zwc.time=thetap.time;
for i=1:N
    p=ptrajectory(thetap.time(i),c,tdata);
    [A,B,C,D]=lpvsys(p);
    R=g^2*eye(nw)-D'*D;
    S=g^2*eye(nz)-D*D';
    theta=thetap.val(i,1:nx)'; p=thetap.val(i,nx+1:end)';
    wwc.val(i,:)=( R\(B'*p+D'*C*theta) )';
    zwc.val(i,:)=( S\(D*B'*p+g^2*C*theta) )';
end;

figure;
subplot(4,1,1); plot(wwc.time,real(wwc.val)); title('real(wwc)');
subplot(4,1,2); plot(wwc.time,imag(wwc.val)); title('imag(wwc)');
subplot(4,1,3); plot(zwc.time,real(zwc.val)); title('real(zwc)');
subplot(4,1,4); plot(zwc.time,imag(zwc.val)); title('imag(zwc)');

% Computing the induced norm
wnormsq=sum(sum(repmat(diff(wwc.time(:)),1,nw).*wwc.val(1:end-1,:).*conj(wwc.val(1:end-1,:))));
znormsq=sum(sum(repmat(diff(zwc.time(:)),1,nz).*zwc.val(1:end-1,:).*conj(zwc.val(1:end-1,:))));
gam=real(sqrt(znormsq/wnormsq))



% Simulating the system for 1 period starting from the initial value
% determind by the eigenvector. (Everything can be complex!)
% -------------------------------------------------------------------
x0=thetap0(1:nx);
[zwc_sim.time,zwc_sim.val]=simlpvsys(lpvsys,ptrajectory,c,tdata,x0,[0 h],wwc.time,wwc.val);
znormsq=sum(sum(repmat(diff(zwc_sim.time(:)),1,nw).*zwc_sim.val(1:end-1,:).*conj(zwc_sim.val(1:end-1,:))));
gam=real(sqrt(znormsq/wnormsq))



% Simulating the system for more periods from zero initial condition by using repeated periods of
% worst-case input. Only the real part of the wc signal is used now.
% --------------------------------------------------------------------

wwc_multi=wwc; 
s=eigval; toffs=wwc_multi.time(end);
for i=1:60
    wwc_multi.val =[wwc_multi.val;wwc.val(2:end,:)*s];
    wwc_multi.time=[wwc_multi.time;wwc.time(2:end)+toffs];    
    s=eigval*s; toffs=wwc_multi.time(end);
end;
%Zeroing the input to get L2 signal
for i=1:30
    wwc_multi.val =[wwc_multi.val;0*wwc.val(2:end,:)];
    wwc_multi.time=[wwc_multi.time;wwc.time(2:end)+toffs];    
    toffs=wwc_multi.time(end);
end;

figure;
subplot(3,1,1); plot(wwc_multi.time,real(wwc_multi.val)); title('real(wwc_multi)');
subplot(3,1,2); plot(wwc_multi.time,imag(wwc_multi.val)); title('imag(wwc_multi)');


%Now, we take the real part of the complex signal
wwc_multi.val=real(wwc_multi.val);



x0=zeros(nx,1);
[zwc_multi.time,zwc_multi.val]=simlpvsys(lpvsys,ptrajectory,c,tdata,x0,[0 wwc_multi.time(end)],wwc_multi.time,wwc_multi.val); 
subplot(3,1,3); plot(zwc_multi.time,zwc_multi.val); title('zwc_multi');

% Computing the norm
znormsq=sum(sum(repmat(diff(zwc_multi.time(:)),1,nw).*zwc_multi.val(1:end-1,:).*conj(zwc_multi.val(1:end-1,:))));
wnormsq=sum(sum(repmat(diff(wwc_multi.time(:)),1,nw).*wwc_multi.val(1:end-1,:).*conj(wwc_multi.val(1:end-1,:))));
gam=real(sqrt(znormsq/wnormsq))


%% Saving the worst-case input and output
save([directory file '_wcio'],'wwc_multi','zwc_multi');




















