clear all; close all;


%% Select an example

example=3;

switch example,
    case 1, 
        lpvsys=@lpvsys_haraldcatws;
        Pbase_xy=@(x,y) [ 1, x, x^2, x^3, y, y^2, y^3 y^4 1/x 1/x^2 y^5];
        Pbase=@(rho) Pbase_xy(rho(1),rho(2));
        Pbase_der_xy=@(x,y,dx,dy) ...
                [  0, dx, 2*dx*x, 3*dx*x^2, dy, 2*dy*y, 3*dy*y^2 4*dy*y^3 -dx/x^2 -2*dx*x/x^4 5*dy*y^4];
        Pbase_der=@(rho,nu) Pbase_der_xy(rho(1),rho(2),nu(1),nu(2));
        M=[30;10]; glb=5.39; solver='mosek'; %solver='sdpt3'; maxiterop='sdpt3.maxit'; maxiter=150;
    case 2, 
        lpvsys=@lpvsys_spinningdisks; lpvsys('init');
        Pbase_xy=@(x,y) [ 1 x, y];% y^2];
        Pbase=@(rho) Pbase_xy(rho(1),rho(2));
        Pbase_der_xy=@(x,y,dx,dy) ...
                [  0 , dx, dy];%, 2*dy*y];
        Pbase_der=@(rho,nu) Pbase_der_xy(rho(1),rho(2),nu(1),nu(2));
        M=[11;11]; glb=1; solver='mosek'; 
        %solver='sdpt3'; maxiter=150;
    case 3, 
        lpvsys=@lpvsys_autopilot; lpvsys('init');
        Pbase_xy=@(x,y) [ 1];%, x, y];
        Pbase=@(rho) Pbase_xy(rho(1),rho(2));
        Pbase_der_xy=@(x,y,dx,dy) ...
                [  0];%, dx, dy];
        Pbase_der=@(rho,nu) Pbase_der_xy(rho(1),rho(2),nu(1),nu(2));
        M=[1;1]; glb=15; solver='mosek'; 
        solver='sdpt3'; maxiter=150;
    
end;



%% The alorithm starts here
[nx,nw,nz,rho_bnds,nu_bnds]=lpvsys([]);
Pbf=Pbase(rho_bnds(:,1));
nofbases=length(Pbf);
if nofbases==1, %here it is assumed that if the number of bases==1, then V is par. independent
    N=ones(size(M)); 
else
    N=2*ones(size(M)); 
end; 
[rho_grid,nu_grid]=mgrid(rho_bnds,M,nu_bnds,N);
sizeofgrd=size(rho_grid,1);




%Constructing the LMI of Bounded Real Lemma
posdef=@(LMI) (LMI>=1e-8*eye(size(LMI)));
negdef=@(LMI) (LMI<=-1e-8*eye(size(LMI)));

P=cell(1,nofbases); Cons=[];
for i=1:nofbases
    P{i}=sdpvar(nx);
end;

g2=sdpvar(1); 
Cons=(g2>=glb^2); 
Obj=g2;

% g2=glb^2;
% Cons=[];
% Obj=[];
    
prgrs=0; prevprgrs=-1; fprintf('Building up the LMIs......');
for k=1:sizeofgrd
    prgrs=fix(100*k/sizeofgrd); 
    if prgrs>prevprgrs, fprintf('\b\b\b\b%3d%%',prgrs); end; 
    prevprgrs=prgrs;
    
    
    rho_k=rho_grid(k,:);
    nu_k =nu_grid(k,:);
    
    [Ak,Bk,Ck,Dk]=lpvsys(rho_k);
    rho_k
    eig(Ak)
    pause
    Pbf=Pbase(rho_k);
    Pbf_der=Pbase_der(rho_k,nu_k);
    
    
    Pk=0; Pkdot=0;
    for i=1:nofbases
        Pk=Pk+Pbf(i)*P{i};
        Pkdot=Pkdot+Pbf_der(i)*P{i};
    end;
    Cons=Cons+posdef(Pk);
    Cons=Cons+negdef([Pkdot+Pk*Ak+Ak'*Pk+Ck'*Ck Pk*Bk+Ck'*Dk; Bk'*Pk+Dk'*Ck Dk'*Dk-g2*eye(nw)]);
end;


fprintf('\nCalling the solver......');
switch solver
    case 'sdpt3',
        options=sdpsettings('verbose',1,'solver','sdpt3','sdpt3.maxit',maxiter);
    case 'sedumi',
        options=sdpsettings('verbose',1,'solver','sedumi','sedumi.maxiter',maxiter);        
    case 'mosek'
        options=sdpsettings('verbose',1,'solver','mosek');
end;
sol=solvesdp(Cons,Obj,options);

if sol.problem ~= 0
    display('Hmm, something went wrong, but we check the results though!');
    sol.info
    %yalmiperror(sol.problem)   
end;
g2=double(g2);
gam=sqrt(g2)
for i=1:nofbases
    P{i}=double(P{i});
end;
    
%Checking the result
for k=1:sizeofgrd
    rho_k=rho_grid(k,:);
    nu_k =nu_grid(k,:);
    
    [Ak,Bk,Ck,Dk]=lpvsys(rho_k);
    Pbf=Pbase(rho_k);
    Pbf_der=Pbase_der(rho_k,nu_k);
    
    
    Pk=0; Pkdot=0;
    for i=1:nofbases
        Pk=Pk+Pbf(i)*P{i};
        Pkdot=Pkdot+Pbf_der(i)*P{i};
    end;
    if any(eig(Pk,'nobalance')<1e-8), error('P is not pos. def'); end;
    if any(eig([Pkdot+Pk*Ak+Ak'*Pk+Ck'*Ck Pk*Bk+Ck'*Dk; Bk'*Pk+Dk'*Ck Dk'*Dk-g2*eye(nw)],'nobalance')>-1e-8) 
        error('Dissip. ineq. is not neg. def.'); end;
end;

    
    



